/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.Phenotype;
import org.fortiss.af3.exploration.model.ExplorationSpecification;
import org.fortiss.af3.exploration.moea.model.DseSpecification;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.TypeLiteral;

/**
 * Provides specifications for a DSE execution to the inner core of the DSE where needed.
 * 
 * @author diewald
 */
public final class InternalSpecificationProvider extends AbstractModule {

	/** Input {@link DseSpecification} . */
	private DseSpecification dseSpec;

	/** Requested result type to be produced by the DSE. */
	private Set<TypeLiteral<? extends Phenotype>> resultTypes;

	/** Constructor taking instances of the input specifications. */
	public InternalSpecificationProvider(DseSpecification dseSpec,
			Set<TypeLiteral<? extends Phenotype>> resultTypes) {
		this.dseSpec = dseSpec;
		this.resultTypes = resultTypes;
	}

	/** Provides the input {@link ExplorationSpecification}. */
	@Provides
	public ExplorationSpecification getExplorationSpecification() {
		return dseSpec.getTargetSpecification();
	}

	/** Provides the input {@link DseSpecification}. */
	@Provides
	public DseSpecification getDseSpecification() {
		return dseSpec;
	}

	/** Provides the requested result type to be produced by the DSE. */
	@Provides
	public Set<TypeLiteral<? extends Phenotype>> getResultTypes() {
		return resultTypes;
	}
}
