/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.create.GenotypeCache;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.PhenotypeCache;

import com.google.inject.AbstractModule;
import com.google.inject.Scope;
import com.google.inject.TypeLiteral;

/**
 * Base class for implementing caches using the {@link Scope} mechanism of Guice. Used in the
 * {@link PhenotypeCache} to implement a thread-local cache, and in the {@link GenotypeCache} for a
 * singleton-based variant.
 * 
 * @author diewald
 */
public class ArtefactCache<T> extends AbstractModule implements Map<TypeLiteral<? extends T>, T> {

	/** Wrapped map. */
	Map<TypeLiteral<? extends T>, T> map = new HashMap<>();

	/** {@inheritDoc} */
	@Override
	public int size() {
		return map.size();
	}

	/** {@inheritDoc} */
	@Override
	public boolean isEmpty() {
		return map.isEmpty();
	}

	/** {@inheritDoc} */
	@Override
	public boolean containsKey(Object key) {
		return map.containsKey(key);
	}

	/** {@inheritDoc} */
	@Override
	public boolean containsValue(Object value) {
		return map.containsValue(value);
	}

	/** {@inheritDoc} */
	@Override
	public T get(Object key) {
		return map.get(key);
	}

	/** {@inheritDoc} */
	@Override
	public T put(TypeLiteral<? extends T> key, T value) {
		return map.put(key, value);
	}

	/** {@inheritDoc} */
	@Override
	public T remove(Object key) {
		return map.remove(key);
	}

	/** {@inheritDoc} */
	@Override
	public void putAll(Map<? extends TypeLiteral<? extends T>, ? extends T> m) {
		map.putAll(m);
	}

	/** {@inheritDoc} */
	@Override
	public void clear() {
		map.clear();
	}

	/** {@inheritDoc} */
	@Override
	public Set<TypeLiteral<? extends T>> keySet() {
		return map.keySet();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<T> values() {
		return map.values();
	}

	/** {@inheritDoc} */
	@Override
	public Set<java.util.Map.Entry<TypeLiteral<? extends T>, T>> entrySet() {
		return map.entrySet();
	}
}
