/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext;

import org.eclipse.jdt.annotation.Nullable;

import com.google.inject.AbstractModule;
import com.google.inject.Injector;
import com.google.inject.Provider;

/**
 * Base class for providing injectors. The {@link Injector} "type" has to be indirectly defined by
 * its provider that is the derived class of this base class.
 * <b>NOTE:</b> It is not possible to implement any {@link InjectorProvider} as an implementation of
 * a {@link Provider} since this causes circular dependencies due to the special role of this
 * interface in Guice. Instead, @Provides methods habe to be used. ONLY the top-most injector is an
 * exception to this rule.
 * 
 * @author diewald
 */
public abstract class InjectorProvider extends AbstractModule {

	/** The {@link Injector} instance to be provided. */
	private Injector injector;

	/** Lazily sets an injector provided by this class. */
	public void setInjector(Injector injector) {
		this.injector = injector;
	}

	/** Returns the {@link Injector} that is hold by this provider. */
	@Nullable
	public Injector getInjector() {
		return injector;
	}
}
