/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.constraint;

import java.util.Collection;

import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;

import com.google.inject.TypeLiteral;

/**
 * Specifies the interface of constraint holders such as associative maps. Internally, artifacts are
 * related to the set of constraints that a producer or modifier has to apply.
 * 
 * @author diewald
 */
public interface IConstrainedArtifactMap {

	/**
	 * Returns the set of {@link ExplorationConstraint}s that have to be applied during construction
	 * of a given artifact type, or the modification of an artifact instance.
	 * 
	 * @param srcType
	 *            Main artifact type to which constraints apply.
	 * @param tgtType
	 *            Secondary artifact type to which constraints apply, such as allocation targets.
	 * @return Set of constraints to apply.
	 */
	Collection<InternalConstraint<?>> getConstraintsOf(TypeLiteral<?> srcType,
			TypeLiteral<?> tgtType);

	/**
	 * Adds the given {@link ExplorationConstraint} to the set set of constraints that shall
	 * be considered during the construction or modification of the given artifact instance or type.
	 * 
	 * @param constraint
	 *            Additional constraint.
	 */
	void addConstraint(InternalConstraint<?> constraint);

	/**
	 * Adds the given set of {@link ExplorationConstraint}s to the set set of constraints that shall
	 * be considered during the construction or modification of the given artifact instance or type.
	 * 
	 * @param constraints
	 *            Set of additional constraints.
	 */
	void addConstraints(Collection<InternalConstraint<?>> constraints);

	/**
	 * Remove all constraints from the given artifact instance or type.
	 * 
	 * @param srcType
	 *            Main artifact type to which constraints apply.
	 * @param tgtType
	 *            Secondary artifact type to which constraints apply, such as allocation targets.
	 */
	void removeArtifact(TypeLiteral<?> srcType, TypeLiteral<?> tgtType);
}
