/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.graph;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.opt4j.core.Genotype;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.google.inject.Binding;
import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.spi.BindingTargetVisitor;

/**
 * Base class to trigger a walk over the bindings of an {@link Injector} using a specialized
 * {@link BindingTargetVisitor} that extracts the relevant information.
 * 
 * @author diewald
 */
public abstract class BindingWalker<T extends BindingTargetVisitor<Object, Collection<Key<?>>>> {

	/** Dependency tree visitor that remembers the visited {@link Genotype} bindings. */
	protected T keyVisitor;

	/** Constructor. */
	public BindingWalker(T keyVisitor) {
		this.keyVisitor = keyVisitor;
	}

	/** Iterates over the bindings for the root keys and their transitive dependencies. */
	protected void iterateBindings(Injector injector, Set<Key<?>> searchKeys) {
		Set<Key<?>> keys = Sets.newHashSet(searchKeys);
		Set<Key<?>> visitedKeys = Sets.newHashSet();
		List<Binding<?>> bindings = Lists.newArrayList();

		while(!keys.isEmpty()) {
			Iterator<Key<?>> iterator = keys.iterator();
			Key<?> key = iterator.next();
			iterator.remove();

			if(!visitedKeys.contains(key)) {
				Binding<?> binding = injector.getBinding(key);
				bindings.add(binding);
				visitedKeys.add(key);
				keys.addAll(binding.acceptTargetVisitor(keyVisitor));
			}
		}
	}
}
