/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.af3.exploration.alg.util.AF3Utils.getAnnotationChecked;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.exception.InvalidModelException;
import org.fortiss.af3.exploration.moea.model.annotation.InterferenceLatency;
import org.fortiss.af3.platform.model.ExecutionUnit;

/**
 * Adapter between AF3 {@link ExecutionUnit} (=deployment target of
 * {@link ITaskAdapter}s) and the DSE.
 * 
 * @author barner
 */
public class AF3ExecutionUnitAdapter extends AF3ResourceAdapter<ExecutionUnit>
		implements IExecutionUnitAdapter {

	/** Additional interference latency introduced by this {@link IExecutionUnitAdapter}. */
	double interferenceLatency = 0;

	/**
	 * Creates a {@link IExecutionUnitAdapter} for an {@link ExecutionUnit}.
	 * 
	 * @throws InvalidModelException
	 *             if a required annotation cannot be found.
	 */
	public AF3ExecutionUnitAdapter(ExecutionUnit executionUnit) {
		super(executionUnit);

		interferenceLatency = getAnnotationChecked(executionUnit, InterferenceLatency.class)
				.getLatency().doubleValue();
	}

	/** {@inheritDoc} */
	@Override
	public double getInterferenceLatency() {
		return interferenceLatency;
	}
}
