/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ISignalAdapter;
import org.jgrapht.graph.DefaultWeightedEdge;

/**
 * Edge specification class for underlying graph implementation.
 * 
 * @author diewald
 */
public class SignalAdapterWeightedEdge extends DefaultWeightedEdge {
	/** {@link ISignalAdapter} represented by this edge. */
	ISignalAdapter signal;

	/** Constructs a new {@link SignalAdapterWeightedEdge} for a given {@link ISignalAdapter}. */
	public SignalAdapterWeightedEdge(ISignalAdapter signal) {
		this.signal = signal;
	}

	/** Returns the {@link ISignalAdapter} associated with this edge. */
	public ISignalAdapter getSignalAdapter() {
		return signal;
	}

	/** {@inheritDoc} */
	@Override
	protected double getWeight() {
		return signal.getSignalSize();
	}
}
