/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.transformation;

import static org.fortiss.af3.allocation.utils.AllocationModelElementFactory.createAllocationEntry;
import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;

import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.partition.variable.Partition;
import org.fortiss.af3.exploration.alg.feature.partition.variable.TaskPartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.ExplorationTransformationModuleBase;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.Transforms;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.partition.model.allocation.TaskToPartitionAllocationEntry;
import org.fortiss.af3.task.model.Task;

import com.google.inject.Provides;

/**
 * Transforms a {@link TaskMappingEncodingBase} to a {@link SuperSet} of AF3
 * {@link TaskToPartitionAllocationEntry}s.
 * 
 * @author diewald
 */
public class TaskToPartitionTransformation
		extends ExplorationTransformationModuleBase<SuperSet<TaskToPartitionAllocationEntry>> {

	/**
	 * See {@link TaskToPartitionTransformation}.
	 * 
	 * @param taskMapping
	 *            allocations of {@link ITaskAdapter}s to {@link Partition}s.
	 * @param af3Partitions
	 *            Required to declare the dependency on the {@link PartitionTransformation}.
	 * @param internalExtAssocMap
	 *            associations between the internal and AF3 partitions.
	 * @return SuperSet of AF3 Partitions.
	 */
	@Transforms
	@Provides
	public <N extends EObject, S extends EObject> SuperSet<TaskToPartitionAllocationEntry>
			transform(TaskPartitionMappingEncoding taskMapping,
					SuperSet<org.fortiss.af3.partition.model.Partition> af3Partitions,
					Map<Partition, org.fortiss.af3.partition.model.Partition> internalExtAssocMap) {
		SuperSet<TaskToPartitionAllocationEntry> allocations =
				createSuperSet("TaskToPart", TaskToPartitionAllocationEntry.class);
		for(TaskMappingEntry<ITaskAdapter, Partition> entry : taskMapping.getMappingEntries()) {
			org.fortiss.af3.partition.model.Partition af3Partition =
					internalExtAssocMap.get(entry.getTarget());
			assert (af3Partition != null) : "Error in the mapping of internal Partitions --> AF3 Partitions.";
			TaskToPartitionAllocationEntry af3AllocEntry =
					createAllocationEntry(TaskToPartitionAllocationEntry.class);
			af3AllocEntry.getSourceElements().add(entry.getSource().getObject(Task.class));
			af3AllocEntry.setTargetElement(af3Partition);
			allocations.getEntries().add(af3AllocEntry);
		}
		return allocations;
	}
}
