/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.partition.variable.operators;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.feature.partition.variable.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.partition.variable.PartitionMappingEntry;

import com.google.inject.Inject;

/**
 * Mutation operator that randomly selects resources from which to remove a random partition. The
 * rate is designed such that one resource is selected in average.
 * 
 * @author diewald
 */
public class MutatePartitionDecrement extends MutatePartitionBase {

	/** Constructor. */
	@Inject
	public MutatePartitionDecrement(Set<IExecutionUnitAdapter> executionUnits) {
		super(executionUnits);
	}

	/** {@inheritDoc} */
	@Override
	public void mutate(PartitionMappingEncoding genotype, double p) {
		Collection<PartitionMappingEntry> preMutateEntries = new ArrayList<>();
		// Consider only entries for which at least two Partitions exist per resource: One partition
		// will always remain.
		for(IExecutionUnitAdapter resource : genotype.getResources()) {
			Collection<PartitionMappingEntry> entries = genotype.getMappingEntriesOf(resource);
			if(entries.size() > 1) {
				preMutateEntries.addAll(entries);
			}
		}

		// Scale the mutation rate with the acutal size of the objects subject to mutation.
		double rate = (p * genotype.size()) / preMutateEntries.size();

		for(PartitionMappingEntry partitionMappingEntry : preMutateEntries) {
			if(rand.nextDouble() < rate) {
				genotype.removeEntry(partitionMappingEntry);
			}
		}
	}
}
