/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.platform;

import java.util.Collection;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.PhenotypeBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Multimap;

/**
 * Encoding that associates parent {@link IExecutionUnitAdapter}s with the templates they contain.
 * Moreover, the contents of the template elements are specified such that they can be instantiated.
 * 
 * @author diewald
 */
public class PlatformExecUnitTemplateEncoding extends PhenotypeBase {

	/**
	 * Associates a parent {@link IExecutionUnitAdapter} with a template
	 * {@link IExecutionUnitAdapter} it contains.
	 */
	private Map<IExecutionUnitAdapter, IExecutionUnitAdapter> parentTemplateMap;

	/**
	 * References all contained {@link IResourceAdapter}s of a template
	 * {@link IExecutionUnitAdapter}.
	 */
	private Multimap<IExecutionUnitAdapter, IResourceAdapter> templateContainedResources;

	/** Constructor. */
	public PlatformExecUnitTemplateEncoding(
			Map<IExecutionUnitAdapter, IExecutionUnitAdapter> parentTemplateMap,
			Multimap<IExecutionUnitAdapter, IResourceAdapter> templateContainedResources) {
		this.parentTemplateMap = parentTemplateMap;
		this.templateContainedResources = templateContainedResources;
	}

	/**
	 * Returns the map that associates a parent {@link IExecutionUnitAdapter} with a template
	 * {@link IExecutionUnitAdapter} it contains.
	 */
	public Collection<IExecutionUnitAdapter> getParentExecUnits() {
		return parentTemplateMap.keySet();
	}

	/**
	 * Returns the template {@link IExecutionUnitAdapter} contained in the given parent
	 * {@link IExecutionUnitAdapter}, or {@code null} if no such element exists.
	 */
	public IExecutionUnitAdapter getTemplateWithin(IExecutionUnitAdapter parentExecUnit) {
		return parentTemplateMap.get(parentExecUnit);
	}

	/**
	 * Returns the {@link IResourceAdapter}s contained in the given template
	 * {@link IExecutionUnitAdapter}.
	 */
	public Collection<IResourceAdapter>
			getContainedResources(IExecutionUnitAdapter templateExecUnit) {
		return templateContainedResources.get(templateExecUnit);
	}

	/** Returns a view on the internal abstract to instances mapping. */
	public ImmutableMap<IExecutionUnitAdapter, IExecutionUnitAdapter> getTemplates() {
		return ImmutableMap.copyOf(parentTemplateMap);
	}
}
