/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.platform.input;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.feature.platform.PlatformExecUnitTemplateEncoding;
import org.fortiss.af3.exploration.alg.module.InputModule;
import org.fortiss.af3.exploration.alg.module.InputProvider;
import org.fortiss.af3.platform.model.ExecutionUnit;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;
import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Maps "parent" {@link ExecutionUnit}s of a hierarchical platform to a contained template
 * {@link ExecutionUnit}.
 * 
 * @author diewald
 */
@Singleton
public class PlatformExecUnitInstanceNumberInputModule
		extends InputModule<ListMultimap<IExecutionUnitAdapter, Integer>> {

	/** Provides the mapping of parents to the templates of their children to instantiate. */
	@InputProvider
	@Provides
	@Singleton
	public ListMultimap<IExecutionUnitAdapter, Integer>
			getAllowedInstances(PlatformExecUnitTemplateEncoding parentChildTemplates) {
		ListMultimap<IExecutionUnitAdapter, Integer> allowedInstances =
				ArrayListMultimap.create();

		for(IExecutionUnitAdapter parentExecUnit : parentChildTemplates.getParentExecUnits()) {
			allowedInstances.put(parentExecUnit, 1);
			allowedInstances.put(parentExecUnit, 2);
			allowedInstances.put(parentExecUnit, 4);
		}

		return allowedInstances;
	}
}
