/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.platform.variable;

import java.util.List;

import org.fortiss.af3.exploration.alg.dse.RandomExploration;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.create.Creates;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.feature.platform.PlatformExecUnitTemplateEncoding;
import org.fortiss.af3.exploration.alg.module.CreatorModule;

import com.google.common.collect.ListMultimap;
import com.google.inject.Provides;

/**
 * Creates an initial, random {@link PlatformExecUnitInstanceEncoding}.
 * 
 * @author diewald
 */
public class PlatformExecUnitInstanceCreator
		extends CreatorModule<PlatformExecUnitInstanceEncoding> {

	/** Random number generator of the D3SE. */
	private RandomExploration rand = RandomExploration.getInstance();

	/**
	 * Creates an {@link PlatformExecUnitInstanceEncoding} where each parent
	 * {@link IExecutionUnitAdapter} is initialized with an allowed number of child
	 * {@link IExecutionUnitAdapter}s.
	 */
	@Creates
	@Provides
	@Genotyped
	public PlatformExecUnitInstanceEncoding create(
			ListMultimap<IExecutionUnitAdapter, Integer> allowedInstances,
			PlatformExecUnitTemplateEncoding templateInstMap) {
		PlatformExecUnitInstanceEncoding encoding =
				new PlatformExecUnitInstanceEncoding(allowedInstances, templateInstMap);

		for(IExecutionUnitAdapter execUnitContainer : encoding.getAbstractContainerExecUnits()) {
			List<Integer> allowedChildInstanceNumbers =
					encoding.getAllowedChildInstances(execUnitContainer);
			int selIdx = rand.nextInt(allowedChildInstanceNumbers.size());
			int numChildExUs = allowedChildInstanceNumbers.get(selIdx);
			encoding.setChildInstanceNumber(execUnitContainer, numChildExUs);
		}

		return encoding;
	}

	/** {@inheritDoc} */
	@Override
	public PlatformExecUnitInstanceEncoding create() {
		// Required by the framework
		return null;
	}
}
