/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.platform.variable;

import java.util.ArrayList;
import java.util.List;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.module.common.operator.mutate.MutateBase;
import org.opt4j.operators.mutate.Mutate;

/**
 * {@link Mutate}s randomly selected elements from the given
 * {@link PlatformExecUnitInstanceEncoding} by assigning a different number of child
 * {@link IExecutionUnitAdapter}s to them.
 * 
 * @author diewald
 */
public class PlatformExecUnitInstanceMutation extends MutateBase<PlatformExecUnitInstanceEncoding> {

	/** {@inheritDoc} */
	@Override
	public void mutate(PlatformExecUnitInstanceEncoding genotype, double p) {
		for(IExecutionUnitAdapter execUnitContainer : genotype.getAbstractContainerExecUnits()) {
			if(rand.nextDouble() < p) {
				List<Integer> allowedChildInstanceNumbers =
						new ArrayList<>(genotype.getAllowedChildInstances(execUnitContainer));
				int selIdx = rand.nextInt(allowedChildInstanceNumbers.size());
				int numChildExUs = allowedChildInstanceNumbers.get(selIdx);
				genotype.setChildInstanceNumber(execUnitContainer, numChildExUs);
			}
		}
	}
}
