/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.safetyarch.problem;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.UpdatesEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.decode.Decodes;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ISafetyFunctionAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.partition.variable.TaskPartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.safetyarch.variable.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.feature.safetyarch.variable.SafetyFunctionArchEntry;
import org.fortiss.af3.exploration.alg.module.DecoderModule;
import org.fortiss.af3.exploration.moea.model.constraints.SeparationConstraint;
import org.opt4j.core.Genotype;

import com.google.inject.Provides;

/**
 * Generates allocation and replication constraints for the safety architectures of the safety
 * function of a given {@link SafetyFunctionArchEncoding}.
 * 
 * @author diewald
 */
public class SFMappingConstraintDecoder<C> extends DecoderModule<TaskPartitionMappingEncoding> {

	/**
	 * Updates the input {@link TaskPartitionMappingEncoding} with additionally required
	 * allocation
	 * entries from the given safety architectures: Additional/Removed elements, replicated
	 * channels.
	 */
	@Provides
	@Decodes
	@UpdatesEncoding
	public TaskPartitionMappingEncoding updateComposable(SafetyFunctionArchEncoding sfaEnc,
			TaskPartitionMappingEncoding atmEnc) {
		for(SafetyFunctionArchEntry sfaEntry : sfaEnc.getAllEntries()) {
			addReplicationConstraints(sfaEntry, sfaEnc);
			addSeparationConstraints(sfaEntry, sfaEnc);
		}

		return atmEnc;
	}

	/**
	 * Add constraints on the number of task replia such that the number of {@link ITaskAdapter}s
	 * corresponds to the number of safety channels.
	 */
	private void addReplicationConstraints(SafetyFunctionArchEntry sFAEntry,
			SafetyFunctionArchEncoding sfaEnc) {
		int channelNumber = sFAEntry.getNumChannels();

		// for(ITaskAdapter comp : sFAEntry.getSafetyFunctionAdapter().getAllComponents()) {
		// sfaEnc.putConstraint(ReplicationConstraint.class,
		// createReplicationConstraint(comp, channelNumber, channelNumber));
		// }
	}

	/**
	 * Creates {@link SeparationConstraint}s for the sets of realizing, diagnostic, and
	 * output units of each safety function.
	 */
	@SuppressWarnings("unchecked")
	private void addSeparationConstraints(SafetyFunctionArchEntry sfaEntry,
			SafetyFunctionArchEncoding sfaEnc) {
		ISafetyFunctionAdapter sfAdp = sfaEntry.getSafetyFunctionAdapter();

		// TODO: We should not be awareCollection<E>le type here.
		// FIXME: Avoid raw casts.
		Collection<Collection<IRequestAdapter>> isoSets = new ArrayList<>();
		isoSets.add((Collection<IRequestAdapter>)(Collection<?>)sfAdp.getRealizingComponents());
		isoSets.add((Collection<IRequestAdapter>)(Collection<?>)sfAdp.getDiagnosticComponents());
		isoSets.add((Collection<IRequestAdapter>)(Collection<?>)sfAdp.getOutputComponent());
		// sfaEnc.putConstraint(SeparationConstraint.class, createSeparationConstraint(isoSets));
	}

	/** {@inheritDoc} */
	@Override
	public TaskPartitionMappingEncoding decode(Genotype genotype) {
		// TODO Auto-generated method stub
		return null;
	}
}
