/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.scheduling;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.Arrays;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;

/**
 * Implementation of an {@link IScheduleElement} using the delegate pattern.
 * 
 * @author diewald
 */
public class ScheduledTask implements InvocationHandler, IScheduleElement {

	/** Actual {@link ITaskAdapter} behind this schedule wrapper. */
	protected ITaskAdapter originTask;

	/** Hiding constructor. */
	private ScheduledTask(ITaskAdapter originTask) {
		this.originTask = originTask;
	}

	/**
	 * Constructs a {@link ScheduledTask} instance.
	 * 
	 * @param originTask
	 *            delegate {@link ITaskAdapter} to be scheduled.
	 * @return a {@link ScheduledTask} instance of the given {@link ITaskAdapter}.
	 */
	public static IScheduleTask createScheduledTask(ITaskAdapter originTask) {
		return (IScheduleTask)Proxy.newProxyInstance(ITaskAdapter.class.getClassLoader(),
				new Class[] {IScheduleTask.class, ITaskAdapter.class},
				new ScheduledTask(originTask));
	}

	/** {@inheritDoc} */
	@Override
	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		Class<?>[] argTypes =
				Arrays.stream(args).map(a -> a.getClass()).toArray(size -> new Class[size]);
		Method delegateMethod = ITaskAdapter.class.getMethod(method.getName(), argTypes);
		if(delegateMethod != null) {
			return method.invoke(originTask, args);
		}
		return method.invoke(this, args);
	}

	/** {@inheritDoc} */
	@Override
	public Double getDeadline() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public Double getOffset() {
		// TODO Auto-generated method stub
		return null;
	}
}
