/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.scheduling;

import org.fortiss.af3.exploration.alg.feature.ExplorationFeature;
import org.fortiss.af3.exploration.alg.feature.scheduling.evaluator.MinLatencyObjectiveEvaluator;
import org.fortiss.af3.exploration.alg.feature.scheduling.evaluator.PeriodConstraintEvaluator;
import org.fortiss.af3.exploration.alg.feature.scheduling.transform.MessageToRouteTransformation;
import org.fortiss.af3.exploration.alg.feature.scheduling.transform.ResourceAllocationTransformation;
import org.fortiss.af3.exploration.moea.model.constraints.PeriodConstraint;
import org.fortiss.af3.exploration.moea.model.objectives.LatencyMinObjective;

import com.google.inject.Injector;
import com.google.inject.TypeLiteral;

/**
 * Decodes a given mapping of task and a mapping of messages (routes) into a time-triggered
 * schedule. First a EDF task scheduling heuristic is applied, before a TDMA message scheduler
 * inserts the slots required for communication. The latter step "stretches" the task schedule.
 * 
 * @author diewald
 */
public class TTScheduleExploration extends ExplorationFeature {

	/** Constructor. */
	public TTScheduleExploration() {
		super("Decoding mappings -> schedules");

		// addProblem(TypeLiteral.get(StrictTTSchedule.class), new StrictTTDecoder());
		addProblem(TypeLiteral.get(StrictTTSchedule.class), new SMTStrictTTDecoder());

		addEvaluator(new PeriodConstraintEvaluator<>(PeriodConstraint.class));
		addEvaluator(new MinLatencyObjectiveEvaluator<>(LatencyMinObjective.class));

		addTransformator(new ResourceAllocationTransformation());
		addTransformator(new MessageToRouteTransformation());
	}

	/** {@inheritDoc} */
	@Override
	public boolean isApplicableToInputs(Injector dseInjector) {
		return true;
	}
}
