/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskinstantiation.variable;

import java.util.HashMap;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.module.common.encoding.InstanceEncodingBase;
import org.fortiss.af3.exploration.alg.module.common.operator.copy.Copyable;

import com.google.common.collect.Multimap;
import com.google.inject.Inject;

/**
 * Encoding that maps partition, which host tasks, to execution units. Typically, these are cores.
 * The encoding is essentially a two-dimensional table.
 * 
 * @author diewald
 */
public class TaskInstanceEncoding extends InstanceEncodingBase<ITaskAdapter>
		implements Copyable<TaskInstanceEncoding> {

	/** References all possible implementations of a given {@link ITaskAdapter}. */
	protected Map<ITaskAdapter, ITaskAdapter> taskInstances = new HashMap<>();

	/** Constructor. */
	@Inject
	public TaskInstanceEncoding(Multimap<ITaskAdapter, ITaskAdapter> instanceMap) {
		this.availableInstances.putAll(instanceMap);
	}

	/** Prototype Constructor. */
	private TaskInstanceEncoding() {
		// Prototype for newInstance().
	}

	/** Copy constructor. */
	protected TaskInstanceEncoding(TaskInstanceEncoding encoding) {
		this.availableInstances.putAll(encoding.availableInstances);
	}

	/** {@inheritDoc} */
	@Override
	public ITaskAdapter getInstanceOf(ITaskAdapter task) {
		return taskInstances.get(task);
	}

	/** {@inheritDoc} */
	@Override
	public ITaskAdapter setInstanceOf(ITaskAdapter task, ITaskAdapter taskInstance) {
		return taskInstances.put(task, taskInstance);
	}

	/** {@inheritDoc} */
	@Override
	public int size() {
		return availableInstances.size();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public TaskInstanceEncoding newInstance() {
		return new TaskInstanceEncoding();
	}

	/** {@inheritDoc} */
	@Override
	public TaskInstanceEncoding copy() {
		return new TaskInstanceEncoding(this);
	}
}
