/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskinstantiation.variable;

import java.util.List;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.module.common.operator.mutate.MutateBase;

/**
 * Randomly assigns a different {@link ITaskAdapter} "instance" for a randomly selected abstract
 * {@link ITaskAdapter}.
 * 
 * @author diewald
 */
public class TaskInstanceMutate extends MutateBase<TaskInstanceEncoding> {

	/** {@inheritDoc} */
	@Override
	public void mutate(TaskInstanceEncoding genotype, double p) {
		for(ITaskAdapter abstractTask : genotype.getAbstractElements()) {
			if(rand.nextDouble() < p) {
				List<ITaskAdapter> availInstances = genotype.getAvailableInstaces(abstractTask);
				int selIdx = rand.nextInt(availInstances.size());
				genotype.setInstanceOf(abstractTask, availInstances.get(selIdx));
			}
		}
	}
}
