/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskmapping;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.PhenotypeBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.module.InputProvider;

import com.google.inject.Provides;

/**
 * Internal encoding that allows to propagate the set of available {@link IExecutionUnitAdapter}s to
 * a task mapping decoder.
 * 
 * @author diewald
 */
public class ExecutionUnitSet extends PhenotypeBase {

	/** Internal {@link IExecutionUnitAdapter} reference. */
	private Set<IExecutionUnitAdapter> executionUnits;

	/** Constructor. */
	public ExecutionUnitSet(Set<IExecutionUnitAdapter> executionUnits) {
		this.executionUnits = executionUnits;
	}

	/** Provides the wrapped set of {@link IExecutionUnitAdapter}s. */
	@InputProvider
	@Provides
	public Set<IExecutionUnitAdapter> getExecutionUnits() {
		return executionUnits;
	}
}
