/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskmapping;

import static java.util.Arrays.asList;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.ExplorationFeature;
import org.fortiss.af3.exploration.alg.feature.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.evaluator.internal.TaskExUAssignmentConstraintEvaluator;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.PlatformCommunicationGraphIdentityDecoder;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.TaskExUMappingIdentityDecoder;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.comm.MessageDecoder;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.comm.MessageEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskExUMappingCreator;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskExUMappingEncoding;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.operator.CrossoverTaskExUMappingModule;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.operator.MutateTaskExUMappingModule;
import org.fortiss.af3.exploration.moea.model.constraints.AssignmentConstraint;

import com.google.inject.Injector;
import com.google.inject.TypeLiteral;

/**
 * Defines a mapping problem of tasks onto a target platform. The default implementation maps onto a
 * bare-metal HW platform. Additional layers such as virtualization has to be realized by additional
 * features.
 * 
 * @author diewald
 */
public class TaskMappingExploration extends ExplorationFeature {

	/** Constructor. */
	public TaskMappingExploration() {
		super("Task Mapping Exploration");

		addVariable(TypeLiteral.get(TaskExUMappingEncoding.class), new TaskExUMappingCreator(),
				asList(new MutateTaskExUMappingModule(), new CrossoverTaskExUMappingModule()));

		addProblem(TypeLiteral.get(ExecutionUnitSet.class), new PlatformExecUnitSetProvider());
		addProblem(TypeLiteral.get(TaskExUMappingEncoding.class),
				new TaskExUMappingIdentityDecoder());
		addProblem(TypeLiteral.get(PlatformCommunicationGraphEncoding.class),
				new PlatformCommunicationGraphIdentityDecoder());
		addProblem(TypeLiteral.get(MessageEncoding.class),
				new MessageDecoder<IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, TaskMappingEncodingBase<ITaskAdapter, IExecutionUnitAdapter, TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, ?>>());

		addEvaluator(new TaskExUAssignmentConstraintEvaluator(AssignmentConstraint.class));

		addTransformator(new TaskToExUTransformation());
	}

	/** {@inheritDoc} */
	@Override
	public boolean isApplicableToInputs(Injector dseInjector) {
		return true;
	}
}
