/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature.taskmapping.variable;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.taskmapping.ExecutionUnitSet;
import org.fortiss.af3.exploration.alg.feature.taskmapping.problem.TaskGraphEncodingBase;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.opt4j.core.Genotype;

/**
 * Defines a mapping of {@link ITaskAdapter}s to physical {@link IExecutionUnitAdapter}s.
 * 
 * @author alex
 */
public class TaskExUMappingEncoding extends
		TaskMappingEncodingBase<ITaskAdapter, IExecutionUnitAdapter, TaskExUMappingEntry, TaskExUMappingEncoding> {

	/** Constructor. */
	protected TaskExUMappingEncoding(ExecutionUnitSet resources,
			TaskGraphEncodingBase<?, ITaskAdapter> tgEncoding,
			Collection<InternalConstraint<?>> constraints, boolean initialize) {
		super(resources.getExecutionUnits(), tgEncoding, constraints, initialize);
	}

	/** Copy Constructor. */
	protected TaskExUMappingEncoding(TaskExUMappingEncoding mapping) {
		super(mapping);
	}

	/** {@inheritDoc} */
	@Override
	public TaskExUMappingEntry createEntry(ITaskAdapter requester,
			IExecutionUnitAdapter resource) {
		return new TaskExUMappingEntry(requester, resource);
	}

	/** {@inheritDoc} */
	@Override
	public <G extends Genotype> G newInstance() {
		// Not needed.
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public TaskExUMappingEncoding copy() {
		return new TaskExUMappingEncoding(this);
	}
}
