/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.opt4j.core.Genotype;
import org.opt4j.core.problem.Creator;

import com.google.common.reflect.TypeToken;
import com.google.inject.AbstractModule;
import com.google.inject.TypeLiteral;

/**
 * {@link Creator} module whose dependencies are defined using guice. Used to define sub-problems
 * of a DSE process for which these modules generate initial values.
 * 
 * @author diewald
 */
public abstract class CreatorModule<G extends Genotype> extends AbstractModule
		implements Creator<G>, IProvidingModule<G>, IConstrainedModule {

	/** Holds the type of the {@link Genotype}s generated by {@code this} CreatorModule. */
	private TypeLiteral<G> genotypeType;

	/** Holds the constraints relevant for the creation of {@link Genotype}s. */
	protected ThreadLocal<Collection<InternalConstraint<?>>> constraints =
			ThreadLocal.withInitial(ArrayList::new);

	/**
	 * Holds the constraints generated during the creation of a {@link Genotype} and targeted at
	 * subsequent create steps.
	 */
	protected ThreadLocal<Collection<InternalConstraint<?>>> generatedContraints =
			ThreadLocal.withInitial(ArrayList::new);

	/** Constructor. */
	@SuppressWarnings("unchecked")
	protected CreatorModule() {
		TypeToken<G> typeToken = new TypeToken<G>(getClass()) {/**/
		};
		genotypeType = (TypeLiteral<G>)TypeLiteral.get(typeToken.getType());
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<G> getProvidedType() {
		return genotypeType;
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<?> getSourceType() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<?> getTargetType() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public void setConstraints(Collection<InternalConstraint<?>> constraints) {
		this.constraints.get().clear();
		this.constraints.get().addAll(constraints);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<InternalConstraint<?>> getGeneratedConstraints() {
		return generatedContraints.get();
	}
}
