/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.phenotype.Phenotype;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.opt4j.core.Genotype;
import org.opt4j.core.problem.Decoder;

import com.google.common.reflect.TypeToken;
import com.google.inject.AbstractModule;
import com.google.inject.TypeLiteral;

/**
 * {@link Decoder} module whose dependencies are defined using guice. Used to define sub-problems
 * of a DSE process.
 * 
 * @author diewald
 */
public abstract class DecoderModule<P extends Phenotype> extends AbstractModule
		implements Decoder<Genotype, P>, IProvidingModule<P>, IConstrainedModule {

	/** Holds the type of the {@link Genotype}s generated by {@code this} CreatorModule. */
	private TypeLiteral<P> phenotypeType;

	/** Holds the constraints relevant for the creation of {@link Phenotype}s. */
	protected ThreadLocal<Collection<InternalConstraint<?>>> constraints =
			ThreadLocal.withInitial(ArrayList::new);
	/**
	 * Holds the constraints generated during the decoding and targeted at subsequent decode steps.
	 */
	protected ThreadLocal<Collection<InternalConstraint<?>>> generatedContraints =
			ThreadLocal.withInitial(ArrayList::new);

	/** Constructor. */
	@SuppressWarnings("unchecked")
	protected DecoderModule() {
		TypeToken<P> typeToken = new TypeToken<P>(getClass()) {/**/
		};
		phenotypeType = (TypeLiteral<P>)TypeLiteral.get(typeToken.getRawType());
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<P> getProvidedType() {
		return phenotypeType;
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<?> getSourceType() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<?> getTargetType() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public void setConstraints(Collection<InternalConstraint<?>> constraints) {
		this.constraints.get().clear();
		this.constraints.get().addAll(constraints);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<InternalConstraint<?>> getGeneratedConstraints() {
		return generatedContraints.get();
	}
}
