/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.service.module.IExplorationEncoding;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;

import com.google.inject.Module;
import com.google.inject.TypeLiteral;

/**
 * Exploration {@link Module}s that implement this interface are able to process constraints such as
 * {@link CreatorModule}s and {@link DecoderModule}s. The {@link IExplorationEncoding}s generated by
 * them shall obey the limits imposed by the passed {@link InternalConstraint}s. Moreover, these
 * modules are also able to generate constraints that can be consumed by other
 * {@link IConstrainedModule}s.
 * 
 * @author diewald
 */
public interface IConstrainedModule {

	/** Returns the primary type of accepted {@link InternalConstraint}s. */
	TypeLiteral<?> getSourceType();

	/**
	 * Returns the secondary type of accepted {@link InternalConstraint}s such as allocation
	 * targets.
	 */
	TypeLiteral<?> getTargetType();

	/** Sets the constraints to be considered by the underlying {@link Module}. */
	void setConstraints(Collection<InternalConstraint<?>> constraints);

	/** Returns the constraints generated by the underlying {@link Module}. */
	Collection<InternalConstraint<?>> getGeneratedConstraints();
}
