/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module;

import org.fortiss.af3.exploration.alg.feature.IExplorationFeature;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;

import com.google.common.reflect.TypeToken;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.TypeLiteral;

/**
 * Base class whose {@link Provides} method annotated with {@link InputProvider} delivers input sets from
 * {@link SuperSet}s to the other D3SE modules. These modules can be overriden by
 * {@link IExplorationFeature}s.
 * 
 * @author diewald
 */
public class InputModule<T> extends AbstractModule implements IProvidingModule<T> {

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public TypeLiteral<T> getProvidedType() {
		TypeToken<T> typeToken = new TypeToken<T>(getClass()) {/**/
		};
		return (TypeLiteral<T>)TypeLiteral.get(typeToken.getType());
	}
}
