/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.encoding;

import java.util.Collection;
import java.util.List;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IModelElementAdapter;
import org.fortiss.af3.exploration.alg.service.module.IExplorationEncoding;
import org.opt4j.core.Genotype;

/**
 * Common interface for {@link IExplorationEncoding}s that instantiate abstract elements.
 * 
 * @author diewald
 */
public interface IInstanceEncoding<T extends IModelElementAdapter>
		extends Genotype, IExplorationEncoding {

	/** Returns the set of abstract elements that need to be instantiated. */
	Collection<T> getAbstractElements();

	/**
	 * Returns the set of available element "implementations" that can be used to instantiate the
	 * given abstract element.
	 */
	List<T> getAvailableInstaces(T abstractElement);

	/** Returns which element is used to instantiate the given abstract element. */
	T getInstanceOf(T abstractElement);

	/**
	 * Sets the given {@code instanceElement} as the instance of the given {@code abstractElement}
	 */
	T setInstanceOf(T instanceTask, T abstractElement);
}
