/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.input;

import static org.fortiss.tooling.common.util.LambdaUtils.filter;
import static org.fortiss.tooling.common.util.LambdaUtils.getFirst;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3.AF3TaskAdapter;
import org.fortiss.af3.exploration.alg.module.InputModule;
import org.fortiss.af3.exploration.alg.module.InputProvider;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.task.model.Task;
import org.fortiss.af3.task.model.allocation.ComponentToTaskAllocationEntry;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Module that provides the {@link ITaskAdapter}s extracted from the source task architecture.
 * 
 * @author diewald
 */
public class DeployableTaskModule extends InputModule<Set<ITaskAdapter>> {

	/** Provides the {@link ITaskAdapter}s of the task architecture. */
	@Provides
	@InputProvider
	@Singleton
	public Set<ITaskAdapter> getTasks(SuperSet<Task> af3Tasks,
			SuperSet<ComponentToTaskAllocationEntry> componentTaskAllocs,
			Set<IExecutionUnitAdapter> execUnits) {
		Set<ITaskAdapter> tasks = new HashSet<>();
		for(Task currentTask : af3Tasks.getEntries()) {
			Optional<ComponentToTaskAllocationEntry> assocComponent =
					getFirst(filter(componentTaskAllocs.getEntries(),
							e -> e.getTargetElement() == currentTask));
			Collection<Component> af3Components = new ArrayList<>();
			assocComponent.ifPresent(e -> af3Components.addAll(e.getComponents()));

			tasks.add(new AF3TaskAdapter(currentTask, af3Components, execUnits));
		}
		return tasks;
	}
}
