/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.input;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IGatewayUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceConnectionAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITransmissionUnitAdapter;
import org.fortiss.af3.exploration.alg.feature.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.module.InputArtifact;
import org.fortiss.af3.exploration.alg.module.InputModule;
import org.fortiss.af3.exploration.alg.module.InputProvider;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * {@link InputModule} that provides the {@link PlatformCommunicationGraphEncoding} of the input
 * system model to other consumers in the D3SE.
 * 
 * @author diewald
 */
public class PlatformCommunicationGraphInputModule
		extends InputModule<PlatformCommunicationGraphEncoding> {

	/**
	 * Provides the {@link PlatformCommunicationGraphEncoding} constructed from the input system
	 * models.
	 */
	@InputProvider
	@Provides
	@Singleton
	@InputArtifact
	public PlatformCommunicationGraphEncoding getPlatformCommunigationGraphEncoding(
			Set<IExecutionUnitAdapter> execUnits, Set<IGatewayUnitAdapter> gatewayUnits,
			Set<ITransmissionUnitAdapter> transmissionUnits,
			Set<IResourceConnectionAdapter> resourceConnections) {
		return new PlatformCommunicationGraphEncoding(execUnits, gatewayUnits, transmissionUnits,
				resourceConnections);
	}
}
