/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.input;

import static java.util.stream.Collectors.toSet;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITransmissionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3.AF3TransmissionUnitAdapter;
import org.fortiss.af3.exploration.alg.module.InputModule;
import org.fortiss.af3.exploration.alg.module.InputProvider;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.af3.platform.model.TransmissionUnit;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Converts the AF3 {@link TransmissionUnit}s into {@link ITransmissionUnitAdapter}s.
 * 
 * @author diewald
 */
public class TransmissionUnitInputModule extends InputModule<Set<ITransmissionUnitAdapter>> {

	/** See {@link TransmissionUnitInputModule}. */
	@InputProvider
	@Provides
	@Singleton
	public Set<ITransmissionUnitAdapter>
			createTransmissionUnits(SuperSet<TransmissionUnit> af3TransMissionUnits) {
		return af3TransMissionUnits.getEntries().stream()
				.map(t -> new AF3TransmissionUnitAdapter(t)).collect(toSet());
	}
}
