/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.operator.crossover.mapping;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;
import org.opt4j.core.Genotype;
import org.opt4j.operators.crossover.Pair;

/**
 * Applies the crossover operation on the allocations of {@link ITaskAdapter}s to
 * {@link IExecutionUnitAdapter}s. It requires two parent {@link TaskMappingEncodingBase}s whose
 * allocations are randomly swapped to generate two new children {@link TaskMappingEncodingBase}s.
 * 
 * @author huang
 * @author diewald
 */
public class CrossoverRandManyAllocation<T extends IExecutionUnitAdapter, E extends TaskMappingEntry<ITaskAdapter, T>, M extends TaskMappingEncodingBase<ITaskAdapter, T, E, ?>>
		extends CrossoverAllocationBase<T, E, M> {

	/**
	 * Probability to swap the allocations of a {@link ITaskAdapter} between two {@link Genotype}s.
	 */
	private double crossoverProb = 0.5;

	/** {@inheritDoc} */
	@Override
	public Pair<M> crossover(M encoding1, M encoding2) {
		boolean appliedCrossover = false;
		for(ITaskAdapter deployableComponent : encoding1.getRequesters()) {
			if(rand.nextDouble() < crossoverProb) {
				crossoverAllocation(deployableComponent, encoding1, encoding2);
			}
		}

		// Check whether the a modification happened, if not enforce one.
		if(!appliedCrossover) {
			int randomIdx = encoding1.getRequesters().size() - 1;
			ITaskAdapter randomlySelectedComponent =
					(ITaskAdapter)encoding1.getRequesters().toArray()[randomIdx];
			crossoverAllocation(randomlySelectedComponent, encoding1, encoding2);
		}

		return new Pair<M>(encoding1, encoding2);
	}
}
