/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.common.operator.mutate.mapping;

import static org.eclipse.core.runtime.Assert.isTrue;

import java.util.List;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEncodingBase;
import org.fortiss.af3.exploration.alg.feature.taskmapping.variable.TaskMappingEntry;

import com.google.inject.Inject;

/**
 * Mutate operator that decreases the redundancy of allocations.
 * 
 * @author huang
 */
public class MutateRedundancyIncrement<S extends TaskMappingEntry<ITaskAdapter, IExecutionUnitAdapter>, T extends TaskMappingEncodingBase<ITaskAdapter, IExecutionUnitAdapter, S, ?>>
		extends MutateRedundancy<S, T> {
	/** Constructs a new increment redundancy mutation operator for a given exploration run */
	@Inject
	public MutateRedundancyIncrement(Set<ITaskAdapter> tasks) {
		super(tasks);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws ExplorationException
	 */
	@Override
	protected void mutate(T encoding, ITaskAdapter deployableComponent, int minReplicatation,
			int maxReplication) throws ExplorationException {
		List<S> allocations = encoding.getMappingEntriesOf(deployableComponent);
		isTrue(allocations != null && !allocations.isEmpty());

		// increase redundancy: the maximum replication should not be exceeded
		if(allocations.size() < maxReplication) {
			// This respects {@link ExcludeDeploymentConstraint}s
			// The following code ensures an equal distribution of temporal and spatial replication
			if(rand.nextBoolean()) {
				S randAllocation = allocations.get(rand.nextInt(allocations.size()));
				encoding.createAndAddEntry(deployableComponent, randAllocation.getTarget());
			} else {
				encoding.createAndAddEntry(deployableComponent, null);
			}
		}
	}
}
