/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.module.expspectransf.constraint;

import static org.fortiss.af3.exploration.alg.util.DesignSpaceExplorationModelElementFactory.createAssignmentConstraint;
import static org.fortiss.af3.exploration.alg.util.DseMLUtils.getDirectAllocationExpression;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers.ExecUnitMapper;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers.TaskMapper;
import org.fortiss.af3.exploration.alg.service.module.base.ExpSpecTransformatorBase;
import org.fortiss.af3.exploration.dseml.model.booleanp.allocation.ILocationExpression;
import org.fortiss.af3.exploration.dseml.model.expression.Set;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.AssignmentConstraint;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.task.model.Task;
import org.fortiss.tooling.base.model.element.IModelElement;

import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.TypeLiteral;

/**
 * Converts an allocation pattern {@link ExplorationConstraint} to an {@link InternalConstraint}.
 * 
 * @author diewald
 */
public final class AllocationPatternConverter
		extends ExpSpecTransformatorBase<ExplorationConstraint<?>, InternalConstraint<?>> {

	/** {@inheritDoc} */
	@Override
	public boolean outputIsReplacement() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public InternalConstraint<?> transform(Injector mainInjector,
			ExplorationConstraint<?> constraint) {
		TaskMapper taskMapper = mainInjector.getInstance(Key.get(new TypeLiteral<TaskMapper>() { // ...
		}));
		ExecUnitMapper execUnitMapper =
				mainInjector.getInstance(Key.get(new TypeLiteral<ExecUnitMapper>() { // ...
				}));

		ILocationExpression locExpression = getDirectAllocationExpression(constraint);
		// TODO(AD): The mappers will not be needed any more when removing the adapters. Then, the
		// code can be also more generic by the set types.
		if(locExpression == null) {
			// No allocation pattern. Exit early.
			return null;
		}

		Set<IModelElement> requesterSet = locExpression.getLeft().getSetReference();
		Set<IModelElement> resourceSet = locExpression.getRight().getSetReference();

		TypeLiteral<?> srcType = TypeLiteral.get(mapAF3Type(requesterSet.getEntryType()));
		TypeLiteral<?> tgtType = TypeLiteral.get(mapAF3Type(resourceSet.getEntryType()));
		AssignmentConstraint assignConstr =
				createAssignmentConstraint(locExpression, srcType, tgtType);

		for(IModelElement me : requesterSet.getEntries()) {
			assignConstr.getSources().add(taskMapper.getAdapter((Task)me));
		}
		for(IModelElement me : resourceSet.getEntries()) {
			assignConstr.getTargets().add(execUnitMapper.getAdapter((ExecutionUnit)me));
		}
		return assignConstr;
	}

	// TODO(AD): Remove or generalize. Obviously, this method should not exist at all...
	/** Translates AF3 types to its internal representation type. */
	private Class<?> mapAF3Type(Class<?> af3Type) {
		if(Task.class.isAssignableFrom(af3Type)) {
			return ITaskAdapter.class;
		} else if(ExecutionUnit.class.isAssignableFrom(af3Type)) {
			return IExecutionUnitAdapter.class;
		}
		return null;
	}
}
