/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.internal.transformation;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.d3seext.ArtefactCache;
import org.fortiss.af3.exploration.alg.service.module.IExplorationTransformationModule;
import org.fortiss.af3.exploration.dseml.model.expression.SuperSet;
import org.fortiss.tooling.base.model.element.IModelElement;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Provides an {@link ArtefactCache} for {@link SuperSet}s that collects all transformed
 * {@link SuperSet}s such that dependent {@link IExplorationTransformationModule}s can use the
 * already transformed {@link SuperSet} instead of reexecuting the transformations.
 * 
 * @author diewald
 */
public class SuperSetCache extends ArtefactCache<SuperSet<? extends IModelElement>> {

	/** Provides a {@link SuperSet} cache per transformation execution. */
	@Provides
	@Singleton
	public SuperSetCache getEncodingCache() {
		return new SuperSetCache();
	}
}
