/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.internal.transformation;

import org.aopalliance.intercept.MethodInterceptor;
import org.fortiss.af3.exploration.alg.service.module.IExplorationTransformationModule;
import org.fortiss.af3.exploration.alg.service.module.base.transformation.Transforms;

import com.google.inject.AbstractModule;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.Provides;
import com.google.inject.matcher.Matchers;

/**
 * {@link Module} that defines the method interceptors of {@link IExplorationTransformationModule}s.
 * This module must be installed at the {@link Injector} that creates the concrete
 * {@link IExplorationTransformationModule}s.
 * Otherwise, the injection is not known at construction time of the
 * {@link IExplorationTransformationModule}s and thus method interception is not operational.
 * 
 * @author diewald
 */
public class TransformationInterceptModule extends AbstractModule {

	/** Reference to the {@link TransformationInterceptor} handling the method interceptions. */
	private TransformationInterceptor transformationInterceptor;

	/** Constructor. The given {@link TransformationInterceptor} is installed with the binder. */
	public TransformationInterceptModule(TransformationInterceptor interceptor) {
		this.transformationInterceptor = interceptor;
	}

	/** Returns the {@link MethodInterceptor} {@link TransformationInterceptor}s. */
	@Provides
	public TransformationInterceptor get() {
		return transformationInterceptor;
	}

	/** {@inheritDoc} */
	@Override
	public void configure() {
		bindInterceptor(Matchers.subclassesOf(IExplorationTransformationModule.class),
				Matchers.annotatedWith(Transforms.class), transformationInterceptor);
	}
}
